package symbolic.satsolving;

import java.util.LinkedList;
import java.util.List;

import jdd.JDD;
import jdd.JDDNode;
import prism.PrismException;
import symbolic.satsolving.AbstractLiteral.Assignment;

public class DirectAND extends DirectAcceptanceGenericDD {

	protected DirectAcceptanceGenericDD left;
	protected DirectAcceptanceGenericDD right;

	public DirectAND(DirectAcceptanceGenericDD left, DirectAcceptanceGenericDD right)
	{
		this.left = left;
		this.right = right;
	}
	
	public DirectAcceptanceGenericDD getLeft()
	{
		return left;
	}
	
	public DirectAcceptanceGenericDD getRight()
	{
		return right;
	}
	
	public String toString()
	{
		return "(" + getLeft() + ") & (" + getRight() + ")";
	}

	@Override
	public List<JDDNode> check(JDDNode mec) throws PrismException {
		List<JDDNode> result = new LinkedList<>();
		// Check, whether left is a unit
		if(getLeft() instanceof DirectLiteral) {
			AbstractLiteral leftAP = ((DirectLiteral)getLeft()).getAP();
			if(leftAP.getKind() == AbstractLiteral.Kind.FIN) {
				List<JDDNode> ecsForRight = getLeft().check(mec);
				
				return getRight().check(ecsForRight);
			}
		}

		// Check, whether right is a unit
		if(getRight() instanceof DirectLiteral) {
			AbstractLiteral rightAP = ((DirectLiteral)getRight()).getAP();
			if(rightAP.getKind() == AbstractLiteral.Kind.FIN) {
				List<JDDNode> ecsForLeft = getRight().check(mec);
				
				return getLeft().check(ecsForLeft);
			}
		}
		
		AbstractLiteral chosenLit =  getFin();
		// Set chosen literal to true
		List<JDDNode> ecs = chosenLit.cutMEC(mec);
		for(JDDNode ec : ecs) {
			List<JDDNode> leftAccEC = getLeft().check(ec);
			result.addAll(getRight().check(leftAccEC));
			JDD.Deref(ec);
		}
		
		// Set chosen literal to false
		return result;
	}

	@Override
	public AbstractLiteral getFin()
	{
		AbstractLiteral ap = getLeft().getFin();

		if(ap != null) {
			return ap;
		}

		return getRight().getFin();
	}

	@Override
	public boolean backtrack(AbstractLiteral literal, boolean value, JDDNode mec) {
		// Recursive backtrack
		boolean resultLeft = getLeft().backtrack(literal, value, mec);
		boolean resultRight = getRight().backtrack(literal, value, mec);
		
		// Nothing has changed for the subformulas, so nothing to do
		if(!resultLeft && !resultRight) {
			return false;
		}

		if(this.value == Assignment.TRUE &&
				(getLeft().getValue() != Assignment.TRUE || getRight().getValue() != Assignment.TRUE)) {
			// if one of the subformulas has a FALSE value, then reset to false, otherwise assign to Unassigned
			this.value = (getLeft().getValue() == Assignment.FALSE || getRight().getValue() == Assignment.FALSE) ?
					Assignment.FALSE : Assignment.UNASSIGNED;
			return true;
		}
		
		if(this.value == Assignment.FALSE && 
				getLeft().getValue() != Assignment.FALSE && getRight().getValue() != Assignment.FALSE) {
			// if both of the subformulas has a TRUE value, then reset to true, otherwise assign to UNassigned
			this.value = (getLeft().getValue() == Assignment.TRUE && getRight().getValue() == Assignment.TRUE) ?
					Assignment.TRUE : Assignment.UNASSIGNED;
			return true;
		}
		
		if(this.value == Assignment.UNASSIGNED &&
				(getLeft().getValue() != Assignment.UNASSIGNED || getLeft().getValue() != Assignment.UNASSIGNED)) {
			// if one of the subformulas has a FALSE value, then reset to false
			if(getLeft().getValue() == Assignment.FALSE || getRight().getValue() == Assignment.FALSE) {
				this.value = Assignment.FALSE;
				return true;
			// if both of the subformulas has a TRUE value, then reset to TRUE
			} else if (getLeft().getValue() == Assignment.TRUE && getRight().getValue() == Assignment.TRUE) {
				this.value = Assignment.TRUE;
				return true;
			}
			// otherwise do nothing
		}
		
		return false;


	}

}