package symbolic.satsolving;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import acceptance.AcceptanceGenericDD;
import jdd.JDD;
import jdd.JDDNode;
import prism.NondetModel;
import prism.NondetModelChecker;
import prism.PrismException;
import symbolic.satsolving.AbstractLiteral.Assignment;

public abstract class DirectAcceptanceGenericDD {
	
	protected Assignment value = Assignment.UNASSIGNED;
	protected List<AbstractLiteral> assignedLiterals = new LinkedList<>();
 
	public Assignment getValue()
	{
		return value;
	}

	public void setValue(Assignment value)
	{
		this.value = value;
	}
	
	public abstract List<JDDNode> check(JDDNode mec) throws PrismException;
	public abstract AbstractLiteral getFin();
	public abstract boolean backtrack(AbstractLiteral literal, boolean value, JDDNode mec);
	
	protected List<JDDNode> check(List<JDDNode> mecs) throws PrismException
	{
		List<JDDNode> result = new LinkedList<>();
		for(JDDNode ec : mecs) {
			result.addAll(check(ec));
		}
		
		JDD.DerefAll(mecs);
		return result;
	}
	
	public static DirectAcceptanceGenericDD createFromAcceptanceGeneric(AcceptanceGenericDD acc, NondetModelChecker mc, NondetModel mdp) throws PrismException
	{
		switch(acc.getKind()) {
		case TRUE:
			return new DirectTrue();
		case FALSE:
			return new DirectFalse();
		case FIN:
		case INF:
			AbstractLiteral ap = acc.getStateAcc() ? new StateLiteral(acc, mc, mdp):
						new TransLiteral(acc, mc, mdp);
			return new DirectLiteral(ap);
		case AND:
			DirectAcceptanceGenericDD left = createFromAcceptanceGeneric(acc.getLeft(), mc, mdp);
			DirectAcceptanceGenericDD right = createFromAcceptanceGeneric(acc.getRight(), mc, mdp);
			
			return new DirectAND(left, right);
		case OR:
			left = createFromAcceptanceGeneric(acc.getLeft(), mc, mdp);
			right = createFromAcceptanceGeneric(acc.getRight(), mc, mdp);
			
			return new DirectOR(left, right);
		default:
			throw new PrismException("Acceptance kind " + acc.getKind() + " is not supported yet");
		}
	}
	
	protected void removeDuplicate(List<JDDNode> ecs)
	{
		JDDNode ec = null;
		Set<JDDNode> seenECs = new LinkedHashSet<>();
		for(Iterator<JDDNode> it = ecs.iterator();
				it.hasNext();
				ec = it.next()) {
			if(seenECs.contains(ec)) {
				JDD.Deref(ec);
				it.remove();
			} else {
				seenECs.add(ec);
			}
		}
	}
}
