package symbolic.satsolving;

import java.util.LinkedList;
import java.util.List;

import jdd.JDD;
import jdd.JDDNode;
import prism.PrismException;
import symbolic.satsolving.AbstractLiteral.Assignment;
import symbolic.satsolving.AbstractLiteral.Kind;

public class DirectLiteral extends DirectAcceptanceGenericDD
{
	protected AbstractLiteral ap = null;
	
	public DirectLiteral(AbstractLiteral ap)
	{
		this.ap = ap;
	}
	
	public AbstractLiteral getAP()
	{
		return ap;
	}
	
	@Override
	public AbstractLiteral getFin()
	{
		if(value == Assignment.UNASSIGNED ||
				ap == null ||
				ap.getKind() != AbstractLiteral.Kind.FIN) {
			return null;
		}
		
		return ap;
	}

	@Override
	public List<JDDNode> check(JDDNode mec) throws PrismException
	{
		// In case of INF ...
		if(ap.getKind() == Kind.INF) {
			// Check for non-emptyness
			// if yes, return mec (satisfied by mec)
			// if no, return empty list (not satisfiable)
			if(!ap.getStates().equals(JDD.ZERO)) {
				List<JDDNode> result = new LinkedList<>();
				result.add(mec.copy());
				value = Assignment.TRUE;
				
				return result;
			} else {
				value = Assignment.FALSE;
				return new LinkedList<>();
			}
		}

		// In case of FIN
		// cut mec with complement,
		// check for subECs
		List<JDDNode> result = ap.cutMEC(mec);
		value = result.isEmpty() ? Assignment.FALSE : Assignment.TRUE;
		return result;
	}

	@Override
	public boolean backtrack(AbstractLiteral literal, boolean value, JDDNode mec) {
		return ap.backtrack(literal, value, mec);
	}

	@Override
	public Assignment getValue()
	{
		if(ap.isUnassigned()) {
			return Assignment.UNASSIGNED;
		}
		
		if(ap.isFalse()) {
			return Assignment.FALSE;
		}
		
		if(ap.isTrue()) {
			return Assignment.TRUE;
		}
		
		return null;
	}
	
	@Override
	public void setValue(Assignment value)
	{
		switch(value) {
		case UNASSIGNED:
			ap.setUnassigned();
			break;
		case FALSE:
			ap.setUnassigned();
			break;
		case TRUE:
			ap.setTrue();
			break;
		}
	}
}
