package symbolic.satsolving;

import java.util.Collections;
import java.util.LinkedList;

import symbolic.satsolving.AbstractLiteral.Kind;

public class LiteralTreeSet extends LinkedList<AbstractLiteral> {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Override
	public boolean add(AbstractLiteral literal) {
		if(literal.getKind() == Kind.SLACKVAR ||
				literal.getKind() == Kind.NEGSLACKVAR) {
			//Construct complement
			AbstractLiteral complLit = new StateLiteral(literal.getIndex(),
					literal.getKind() == Kind.NEGSLACKVAR);
			if(contains(complLit) || contains(literal)) {
				return false;
			}
		}
		
		Collections.sort(this);
		
		if(literal.getKind() == Kind.NEGSLACKVAR) {
			AbstractLiteral complement = new StateLiteral(literal.getIndex(), false);
			insert(complement);
			return true;
		}

		insert(literal);
		return true;
	}
	
	@Override
	public boolean remove(Object o) {
		if(!(o instanceof AbstractLiteral)) {
			return false;
		}
		
		AbstractLiteral literal = (AbstractLiteral)o;
		if(literal.getKind() == Kind.NEGSLACKVAR) {
			return super.remove(new StateLiteral(literal.getIndex(), false));
		}
		return super.remove(literal);
	}
	
	private void insertInterval(AbstractLiteral e, int firstPos, int lastPos)
	{
		if(e.compareTo(get(firstPos)) < 0) {
			add(firstPos, e);
			return;
		} 
		
		if(e.compareTo(get(lastPos)) > 0) {
			add(lastPos+1, e);
			return;
		}
		
		int middle = (firstPos + lastPos)/2;
		AbstractLiteral midLit = get(middle);
		
		if(e.compareTo(midLit) < 0) {
			insertInterval(e, firstPos+1, middle);
			return;
		}
		
		if(e.compareTo(midLit) > 0) {
			insertInterval(e, middle, lastPos - 1);
		}
	}
	
	private void insert(AbstractLiteral e)
	{
		if(isEmpty()) {
			super.add(e);
		} else {
			insertInterval(e, 0, size()-1);
		}
	}
}