package symbolic.satsolving;

import java.util.ArrayList;
import java.util.List;

import acceptance.AcceptanceGenericDD;
import jdd.JDD;
import jdd.JDDNode;
import prism.NondetModel;
import prism.NondetModelChecker;
import prism.PrismException;

public class StateCNF extends CNF {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5962843027221255533L;

	/**
	 * Constructor for StateCNF
	 * @param acc the generic acceptance, for which a CNF should be create
	 * @param mdp the MDP, which should be analysed under acc
	 * @param mc the according MDPModelChecker
	 * @param implicationGraph the current implicationGraph
	 * @throws PrismException
	 */
	public StateCNF(AcceptanceGenericDD acc, NondetModel mdp, NondetModelChecker mc) throws PrismException
	{
		super(acc,  mdp, mc);
	}

	@Override
	protected AbstractLiteral createLiteral(JDDNode node, boolean inf) {
		return new StateLiteral(node, inf, mc, mdp);
	}

	@Override
	protected AbstractLiteral createLiteral(int index, boolean negated) {
		return new StateLiteral(index, negated);
	}

	@Override
	protected AbstractLiteral createLiteral(AcceptanceGenericDD acc) throws PrismException {
		return new StateLiteral(acc, mc, mdp);
	}

	/**
	 * If literal is Fin(P), then remove P from mecs, and generate a new maximal end component within mecs\P
	 * @param mecs the maximal end component, which should be pruned
	 * @param literal the literal saying which states should be removed
	 * @return the maximal end components within mecs\P
	 * @throws PrismException
	 */
	@Override
	public List<JDDNode> cutMEC(JDDNode mecs, AbstractLiteral literal) throws PrismException
	{
		if(literal.getKind() == AbstractLiteral.Kind.FIN) {
			JDDNode cuttedEC = JDD.And(mecs.copy(),
					JDD.Not(literal.getStates().copy()));
			List<JDDNode> result = getMECs(cuttedEC, null);
			JDD.Deref(cuttedEC);
			return result;
		}
		
		//Nothing to do for literal.kind != FIN
		List<JDDNode> id = new ArrayList<>(1);
		id.add(mecs.copy());
		return id;
	}

}
