package symbolic.satsolving;

import java.util.ArrayList;
import java.util.List;

import acceptance.AcceptanceGenericDD;
import jdd.JDD;
import jdd.JDDNode;
import prism.NondetModel;
import prism.PrismComponent;
import prism.PrismException;

public class StateLiteral extends AbstractLiteral {

	public StateLiteral(AcceptanceGenericDD acc, PrismComponent mc, NondetModel mdp) throws PrismException {
		super(acc, mc, mdp);
		this.numVars = mdp.getNumDDRowVars();
	}

	public StateLiteral(int index, boolean negated) 
	{
		super(index, negated);
	}
	
	public StateLiteral(JDDNode states, boolean inf, PrismComponent mc, NondetModel mdp)
	{
		super(states, inf, mc, mdp);
		this.numVars = mdp.getNumDDRowVars();
	}

	@Override
	public void cutWithMEC(JDDNode mec) 
	{
		if(getKind() == AbstractLiteral.Kind.INF || getKind() == AbstractLiteral.Kind.FIN) {
			setStates(JDD.And(getStates(),
					mec.copy()));
			if(getStates().equals(JDD.ZERO)) {
				if(getKind() == AbstractLiteral.Kind.INF) {
					setFalse();
				}
				if(getKind() == AbstractLiteral.Kind.FIN) {
					setTrue();
				}
			}
		}
	}

	/**
	 * Returns a deep copy of this
	 */
	@Override
	public AbstractLiteral clone()
	{
		AbstractLiteral literal = null;
		switch(kind) {
		case INF:
			literal = new StateLiteral(getStates(), true, mc, mdp);
			break;
		case FIN:
			literal = new StateLiteral(getStates(), false, mc, mdp);
			break;
		case SLACKVAR:
			literal = new StateLiteral(index, false);
			break;
		case NEGSLACKVAR:
			literal = new StateLiteral(index, true);
			break;
		default:
				
		}
		literal.value = value;
		return literal;
	}

	@Override
	public StateLiteral complement()
	{
		switch(getKind()) {
		case SLACKVAR:
			return new StateLiteral(getIndex(), true);
		case NEGSLACKVAR:
			return new StateLiteral(getIndex(), false);
		case INF:
			return new StateLiteral(getStates(), false, mc, mdp);
		case FIN:
			return new StateLiteral(getStates(), true, mc, mdp);
		default:
			return null;
		}
	}

	/**
	 * If literal is Fin(P), then remove P from mecs, and generate a new maximal end component within mecs\P
	 * @param mecs the maximal end component, which should be pruned
	 * @return the maximal end components within mecs\P
	 * @throws PrismException
	 */
	@Override
	public List<JDDNode> cutMEC(JDDNode mecs) throws PrismException
	{
		if(getKind() == AbstractLiteral.Kind.FIN) {
			JDDNode cuttedEC = JDD.And(mecs.copy(),
					JDD.Not(getStates().copy()));
			List<JDDNode> result = getMECs(cuttedEC, null);
			JDD.Deref(cuttedEC);
			return result;
		}
		
		//Nothing to do for literal.kind != FIN
		List<JDDNode> id = new ArrayList<>(1);
		id.add(mecs.copy());
		return id;
	}

}
